@@ -123,8 +123,8 @@
 /* .PP
 /*	Available in Postfix version 3.2 and later:
 /* .IP "\fBtls_eecdh_auto_curves (see 'postconf -d' output)\fR"
-/*	The prioritized list of elliptic curves supported by the Postfix
-/*	SMTP client and server.
+/*	The prioritized list of elliptic curves, that should be enabled in the
+/*	Postfix SMTP client and server.
 /* .PP
 /*	Available in Postfix version 3.4 and later:
 /* .IP "\fBtls_server_sni_maps (empty)\fR"
@@ -738,12 +738,12 @@ static int tlsp_eval_tls_error(TLSP_STATE *state, int err)
 	 * already reported.
 	 */
 #ifdef USE_TLSRPT
-	if (state->client_start_props->tlsrpt
+	if (state->is_server_role == 0
 	    && (state->flags & TLSP_FLAG_DO_HANDSHAKE)
-	    && state->is_server_role == 0)
+	    && state->client_start_props->tlsrpt)
 	    trw_report_failure(state->client_start_props->tlsrpt,
 			       TLSRPT_VALIDATION_FAILURE,
-			        /* additional_detail= */ (char *) 0,
+			        /* additional_info= */ (char *) 0,
 			       "tls-handshake-failure");
 #endif
 	tlsp_state_free(state);
@@ -1690,6 +1690,12 @@ static void pre_jail_init_client(void)
      */
     tlsp_client_app_cache = htable_create(10);
 
+    /* Postfix <= 3.10 backwards compatibility. */
+    if (warn_compat_break_tlsp_clnt_level
+	&& warn_compat_break_smtp_tls_level)
+	msg_info("using backwards-compatible default setting "
+		 VAR_TLSP_CLNT_LEVEL "=(empty)");
+
     /*
      * Most sites don't use TLS client certs/keys. In that case, enabling
      * tlsproxy-based connection caching is trivial.
