@@ -60,10 +60,6 @@
 /*	Request that match_list_match() logs a warning and returns
 /*	zero (with list->error set to a non-zero dictionary error
 /*	code) instead of raising a fatal run-time error.
-/* .IP MATCH_FLAG_NOFILE
-/*	Disable special handling for /file/name.
-/* .IP MATCH_FLAG_NODICT
-/*	Disable special handling for type:name.
 /* .RE
 /*	Specify MATCH_FLAG_NONE to request none of the above.
 /* .IP pattern_list
@@ -88,9 +84,6 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
-/*
-/*	Wietse Venema
-/*	porcupine.org
 /*--*/
 
 /* System library. */
@@ -159,8 +152,7 @@ static ARGV *match_list_parse(MATCH_LIST *match_list, ARGV *pat_list,
 	if (*item == 0)
 	    /* No graceful degradation for this... */
 	    msg_fatal("%s: no pattern after '!'", match_list->pname);
-	if (*item == '/'			/* /file/name */
-	    && (match_list->flags & MATCH_FLAG_NOFILE) == 0) {
+	if (*item == '/') {			/* /file/name */
 	    if ((fp = vstream_fopen(item, O_RDONLY, 0)) == 0) {
 		/* Replace unusable pattern with pseudo table. */
 		vstring_sprintf(buf, "%s:%s", DICT_TYPE_NOFILE, item);
@@ -177,14 +169,13 @@ static ARGV *match_list_parse(MATCH_LIST *match_list, ARGV *pat_list,
 		if (vstream_fclose(fp))
 		    msg_fatal("%s: read file %s: %m", myname, item);
 	    }
-	} else if (MATCH_DICTIONARY(item)	/* type:table */
-		   &&(match_list->flags & MATCH_FLAG_NODICT) == 0) {
-	    vstring_sprintf(buf, "%s%s(%o,%s)", match ? "" : "!",
-			    item, OPEN_FLAGS, dict_flags_str(DICT_FLAGS));
-	    map_type_name_flags = STR(buf) + (match == 0);
+	} else if (MATCH_DICTIONARY(item)) {	/* type:table */
+	    map_type_name_flags = dict_make_registered_name(buf, item, OPEN_FLAGS,
+							    DICT_FLAGS);
 	    if (dict_handle(map_type_name_flags) == 0)
-		dict_register(map_type_name_flags,
-			      dict_open(item, OPEN_FLAGS, DICT_FLAGS));
+		dict_open(item, OPEN_FLAGS, DICT_FLAGS);
+	    if (match == 0)
+		vstring_prepend(buf, "!", 1);
 	    argv_add(pat_list, STR(buf), (char *) 0);
 	} else {				/* other pattern */
 	    casefold(match_list->fold_buf, match ?
